@@ -55,7 +55,7 @@ pub(crate) fn ghost_col_etree<'n, I: Index>(
 	}
 }
 
-/// computes the size and alignment of the workspace required to compute the column elimination tree
+/// computes the layout of the workspace required to compute the column elimination tree
 /// of a matrix $A$ with dimensions `(nrows, ncols)`
 #[inline]
 pub fn col_etree_scratch<I: Index>(nrows: usize, ncols: usize) -> StackReq {
@@ -249,10 +249,10 @@ pub(crate) fn ghost_column_counts_aat<'m, 'n, I: Index>(
 	}
 }
 
-/// computes the size and alignment of the workspace required to compute the column counts
+/// computes the layout of the workspace required to compute the column counts
 /// of the cholesky factor of the matrix $A A^\top$, where $A$ has dimensions `(nrows, ncols)`
 #[inline]
-pub fn column_counts_aat_scrach<I: Index>(nrows: usize, ncols: usize) -> StackReq {
+pub fn column_counts_aat_scratch<I: Index>(nrows: usize, ncols: usize) -> StackReq {
 	StackReq::all_of(&[StackReq::new::<I>(nrows).array(5), StackReq::new::<I>(ncols)])
 }
 
@@ -290,7 +290,7 @@ pub fn column_counts_ata<'m, 'n, I: Index>(
 	)
 }
 
-/// computes the size and alignment of the workspace required to compute the postordering of an
+/// computes the layout of the workspace required to compute the postordering of an
 /// elimination tree of size `n`
 #[inline]
 pub fn postorder_scratch<I: Index>(n: usize) -> StackReq {
@@ -325,7 +325,7 @@ pub mod supernodal {
 		col_ptr_for_tau_val: alloc::vec::Vec<I>,
 		col_ptr_for_val: alloc::vec::Vec<I>,
 		super_etree: alloc::vec::Vec<I>,
-		max_blocksize: alloc::vec::Vec<I>,
+		max_block_size: alloc::vec::Vec<I>,
 		nrows: usize,
 	}
 
@@ -406,7 +406,7 @@ pub mod supernodal {
 			&self.H
 		}
 
-		/// computes the size and alignment of the workspace required to solve the linear system
+		/// computes the layout of the workspace required to solve the linear system
 		/// $A x = \text{rhs}$ in the sense of least squares
 		pub fn solve_in_place_scratch<T: ComplexField>(&self, rhs_ncols: usize, par: Par) -> StackReq {
 			let _ = par;
@@ -418,12 +418,12 @@ pub mod supernodal {
 			for s in 0..n_supernodes {
 				let s_h_row_begin = H_symbolic.col_ptr_for_row_idx[s].zx();
 				let s_h_row_full_end = H_symbolic.col_ptr_for_row_idx[s + 1].zx();
-				let max_blocksize = H_symbolic.max_blocksize[s].zx();
+				let max_block_size = H_symbolic.max_block_size[s].zx();
 
 				loop_scratch = loop_scratch.or(
 					linalg::householder::apply_block_householder_sequence_transpose_on_the_left_in_place_scratch::<T>(
 						s_h_row_full_end - s_h_row_begin,
-						max_blocksize,
+						max_block_size,
 						rhs_ncols,
 					),
 				);
@@ -433,7 +433,7 @@ pub mod supernodal {
 		}
 	}
 
-	/// computes the size and alignment of the workspace required to compute the symbolic $QR$
+	/// computes the layout of the workspace required to compute the symbolic $QR$
 	/// factorization of a matrix with dimensions `(nrows, ncols)`
 	pub fn factorize_supernodal_symbolic_qr_scratch<I: Index>(nrows: usize, ncols: usize) -> StackReq {
 		let _ = nrows;
@@ -530,7 +530,7 @@ pub mod supernodal {
 		let mut col_ptr_for_tau_val = try_zeroed::<I>(n_supernodes + 1)?;
 		let mut col_ptr_for_val = try_zeroed::<I>(n_supernodes + 1)?;
 		let mut super_etree_ = try_zeroed::<I>(n_supernodes)?;
-		let mut max_blocksize = try_zeroed::<I>(n_supernodes)?;
+		let mut max_block_size = try_zeroed::<I>(n_supernodes)?;
 		let super_etree = bytemuck::cast_slice_mut::<I, I::Signed>(&mut super_etree_);
 
 		let to_wide = |i: I| i.zx() as u128;
@@ -595,9 +595,9 @@ pub mod supernodal {
 			let s_col_count = panel_width + (L_col_ptr_for_row_idx[*s + 1] - L_col_ptr_for_row_idx[*s]);
 			val_count += to_wide(s_row_count) * to_wide(s_col_count);
 			row_count += to_wide(s_row_count);
-			let blocksize = linalg::qr::no_pivoting::factor::recommended_blocksize::<Symbolic>(s_row_count.zx(), s_col_count.zx()) as u128;
-			max_blocksize[*s] = from_wide(blocksize);
-			tau_count += blocksize * to_wide(Ord::min(s_row_count, s_col_count));
+			let block_size = linalg::qr::no_pivoting::factor::recommended_block_size::<Symbolic>(s_row_count.zx(), s_col_count.zx()) as u128;
+			max_block_size[*s] = from_wide(block_size);
+			tau_count += block_size * to_wide(Ord::min(s_row_count, s_col_count));
 			*next_val_ptr = from_wide(val_count);
 			*next_row_ptr = from_wide(row_count);
 			*next_tau_ptr = from_wide(tau_count);
@@ -611,7 +611,7 @@ pub mod supernodal {
 			col_ptr_for_val,
 			super_etree: super_etree_,
 			col_ptr_for_tau_val,
-			max_blocksize,
+			max_block_size,
 			nrows: *M,
 		})
 	}
@@ -624,7 +624,7 @@ pub mod supernodal {
 		householder_val: &'a [T],
 		tau_val: &'a [T],
 		householder_row_idx: &'a [I],
-		tau_blocksize: &'a [I],
+		tau_block_size: &'a [I],
 		householder_nrows: &'a [I],
 		householder_ncols: &'a [I],
 	}
@@ -646,7 +646,7 @@ pub mod supernodal {
 		pub unsafe fn new_unchecked(
 			symbolic: &'a SymbolicSupernodalQr<I>,
 			householder_row_idx: &'a [I],
-			tau_blocksize: &'a [I],
+			tau_block_size: &'a [I],
 			householder_nrows: &'a [I],
 			householder_ncols: &'a [I],
 			r_val: &'a [T],
@@ -659,10 +659,10 @@ pub mod supernodal {
 			assert!(rt_val.len() == symbolic.R_adjoint().len_val());
 			assert!(tau_val.len() == symbolic.householder().len_tau_val());
 			assert!(householder_val.len() == symbolic.householder().len_householder_val());
-			assert!(tau_blocksize.len() == householder_nrows.len());
+			assert!(tau_block_size.len() == householder_nrows.len());
 			Self {
 				symbolic,
-				tau_blocksize,
+				tau_block_size,
 				householder_nrows,
 				householder_ncols,
 				rt_val,
@@ -696,26 +696,23 @@ pub mod supernodal {
 			self.tau_val
 		}
 
-		/// solves the equation $A x = \text{rhs}$ in the sense of least squares, implicitly
-		/// conjugating $A$ if needed
+		/// Applies $Q^{\top}$ to the rhs in place, implicitly conjugating $Q$ if needed
 		///
 		/// `work` is a temporary workspace with the same dimensions as `rhs`
-		#[track_caller]
 		#[math]
-		pub fn solve_in_place_with_conj(&self, conj: Conj, rhs: MatMut<'_, T>, par: Par, work: MatMut<'_, T>, stack: &mut MemStack)
+		pub fn apply_Q_transpose_in_place_with_conj(&self, conj: Conj, rhs: MatMut<'_, T>, par: Par, work: MatMut<'_, T>, stack: &mut MemStack)
 		where
 			T: ComplexField,
 		{
 			let L_symbolic = self.symbolic().R_adjoint();
 			let H_symbolic = self.symbolic().householder();
 			let n_supernodes = L_symbolic.n_supernodes();
+			let mut stack = stack;
 
 			assert!(rhs.nrows() == self.symbolic().householder().nrows);
 
 			let mut x = rhs;
 			let k = x.ncols();
-
-			let mut stack = stack;
 			let mut tmp = work;
 			tmp.fill(zero());
 
@@ -742,7 +739,7 @@ pub mod supernodal {
 					for j in 0..k {
 						for idx in 0..s_h_row_full_end - s_h_row_begin {
 							let i = s_row_idx_in_panel[idx].zx();
-							tmp.write(idx, j, x.read(i, j));
+							tmp[(idx, j)] = copy(x[(i, j)]);
 						}
 					}
 
@@ -754,13 +751,13 @@ pub mod supernodal {
 						s_ncols + (L_symbolic.col_ptr_for_row_idx()[s + 1].zx() - L_symbolic.col_ptr_for_row_idx()[s].zx()),
 					);
 					let s_tau = &tau[tau_begin..tau_end];
-					let max_blocksize = H_symbolic.max_blocksize[s].zx();
-					let s_tau = MatRef::from_column_major_slice(s_tau, max_blocksize, Ord::min(s_H.ncols(), s_h_row_full_end - s_h_row_begin));
+					let max_block_size = H_symbolic.max_block_size[s].zx();
+					let s_tau = MatRef::from_column_major_slice(s_tau, max_block_size, Ord::min(s_H.ncols(), s_h_row_full_end - s_h_row_begin));
 
 					let mut start = 0;
 					let end = s_H.ncols();
 					while start < end {
-						let bs = self.tau_blocksize[block_count].zx();
+						let bs = self.tau_block_size[block_count].zx();
 						let nrows = self.householder_nrows[block_count].zx();
 						let ncols = self.householder_ncols[block_count].zx();
 
@@ -770,7 +767,7 @@ pub mod supernodal {
 						linalg::householder::apply_block_householder_sequence_transpose_on_the_left_in_place_with_conj(
 							b_H.rb(),
 							b_tau.rb(),
-							Conj::Yes.compose(conj),
+							conj,
 							tmp.rb_mut().subrows_mut(start, nrows),
 							par,
 							stack.rb_mut(),
@@ -787,7 +784,7 @@ pub mod supernodal {
 					for j in 0..k {
 						for idx in 0..s_h_row_full_end - s_h_row_begin {
 							let i = s_row_idx_in_panel[idx].zx();
-							x.write(i, j, tmp.read(idx, j));
+							x[(i, j)] = copy(tmp[(idx, j)]);
 						}
 					}
 				}
@@ -796,6 +793,28 @@ pub mod supernodal {
 			let n = L_symbolic.nrows();
 			x.rb_mut().subrows_mut(0, n).copy_from(tmp.rb().subrows(0, n));
 			x.rb_mut().subrows_mut(n, m - n).fill(zero());
+		}
+
+		/// solves the equation $A x = \text{rhs}$ in the sense of least squares, implicitly
+		/// conjugating $A$ if needed
+		///
+		/// `work` is a temporary workspace with the same dimensions as `rhs`
+		#[track_caller]
+		#[math]
+		pub fn solve_in_place_with_conj(&self, conj: Conj, rhs: MatMut<'_, T>, par: Par, work: MatMut<'_, T>, stack: &mut MemStack)
+		where
+			T: ComplexField,
+		{
+			let mut work = work;
+			let mut rhs = rhs;
+			self.apply_Q_transpose_in_place_with_conj(conj.compose(Conj::Yes), rhs.rb_mut(), par, work.rb_mut(), stack);
+
+			let L_symbolic = self.symbolic().R_adjoint();
+			let n_supernodes = L_symbolic.n_supernodes();
+
+			let mut tmp = work;
+			let mut x = rhs;
+			let k = x.ncols();
 
 			// x <- R^-1 x = L^-T x
 			{
@@ -811,7 +830,7 @@ pub mod supernodal {
 					for j in 0..k {
 						for (idx, i) in s.pattern().iter().enumerate() {
 							let i = i.zx();
-							tmp.write(idx, j, x.read(i, j));
+							tmp[(idx, j)] = copy(x[(i, j)]);
 						}
 					}
 
@@ -837,7 +856,7 @@ pub mod supernodal {
 		}
 	}
 
-	/// computes the size and alignment of the workspace required to compute the numerical $QR$
+	/// computes the layout of the workspace required to compute the numerical $QR$
 	/// factorization of the matrix whose structure was used to produce the symbolic structure
 	#[track_caller]
 	pub fn factorize_supernodal_numeric_qr_scratch<I: Index, T: ComplexField>(
@@ -861,7 +880,7 @@ pub mod supernodal {
 		for s in 0..n_supernodes {
 			let s_h_row_begin = symbolic.H.col_ptr_for_row_idx[s].zx();
 			let s_h_row_full_end = symbolic.H.col_ptr_for_row_idx[s + 1].zx();
-			let max_blocksize = symbolic.H.max_blocksize[s].zx();
+			let max_block_size = symbolic.H.max_block_size[s].zx();
 			let s_col_begin = symbolic.L.supernode_begin()[s].zx();
 			let s_col_end = symbolic.L.supernode_end()[s].zx();
 			let s_ncols = s_col_end - s_col_begin;
@@ -870,7 +889,7 @@ pub mod supernodal {
 			loop_scratch = loop_scratch.or(linalg::qr::no_pivoting::factor::qr_in_place_scratch::<T>(
 				s_h_row_full_end - s_h_row_begin,
 				s_ncols + s_pattern_len,
-				max_blocksize,
+				max_block_size,
 				par,
 				params,
 			));
@@ -878,7 +897,7 @@ pub mod supernodal {
 			loop_scratch = loop_scratch.or(
 				linalg::householder::apply_block_householder_sequence_transpose_on_the_left_in_place_scratch::<T>(
 					s_h_row_full_end - s_h_row_begin,
-					max_blocksize,
+					max_block_size,
 					s_ncols + s_pattern_len,
 				),
 			);
@@ -890,7 +909,7 @@ pub mod supernodal {
 	/// computes the numerical $QR$ factorization of $A$
 	///
 	/// - `householder_row_idx` must have length `symbolic.householder().len_householder_row_idx()`
-	/// - `tau_blocksize` must have length `symbolic.householder().len_householder_row_idx() +
+	/// - `tau_block_size` must have length `symbolic.householder().len_householder_row_idx() +
 	///   symbolic.householder().n_supernodes()`
 	/// - `householder_nrows` must have length `symbolic.householder().len_householder_row_idx()
 	///   + symbolic.householder().n_supernodes()`
@@ -905,7 +924,7 @@ pub mod supernodal {
 	#[track_caller]
 	pub fn factorize_supernodal_numeric_qr<'a, I: Index, T: ComplexField>(
 		householder_row_idx: &'a mut [I],
-		tau_blocksize: &'a mut [I],
+		tau_block_size: &'a mut [I],
 		householder_nrows: &'a mut [I],
 		householder_ncols: &'a mut [I],
 
@@ -925,14 +944,14 @@ pub mod supernodal {
 			r_val.len() == symbolic.R_adjoint().len_val(),
 			householder_val.len() == symbolic.householder().len_householder_val(),
 			tau_val.len() == symbolic.householder().len_tau_val(),
-			tau_blocksize.len() == symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes(),
+			tau_block_size.len() == symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes(),
 			householder_nrows.len() == symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes(),
 			householder_ncols.len() == symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes(),
 		));
 
 		factorize_supernodal_numeric_qr_impl(
 			householder_row_idx,
-			tau_blocksize,
+			tau_block_size,
 			householder_nrows,
 			householder_ncols,
 			r_val,
@@ -956,7 +975,7 @@ pub mod supernodal {
 			SupernodalQrRef::<'_, I, T>::new_unchecked(
 				symbolic,
 				householder_row_idx,
-				tau_blocksize,
+				tau_block_size,
 				householder_nrows,
 				householder_ncols,
 				r_val,
@@ -971,7 +990,7 @@ pub mod supernodal {
 		// len: col_ptr_for_row_idx[n_supernodes]
 		householder_row_idx: &mut [I],
 
-		tau_blocksize: &mut [I],
+		tau_block_size: &mut [I],
 		householder_nrows: &mut [I],
 		householder_ncols: &mut [I],
 
@@ -1106,7 +1125,7 @@ pub mod supernodal {
 						let pj = col_perm.map(|perm| perm.arrays().1[j].zx()).unwrap_or(j);
 						let ix = idx;
 						let iy = col_global_to_local[pj].zx();
-						s_H.write(ix, iy, s_H[(ix, iy)] + *value);
+						s_H[(ix, iy)] = s_H[(ix, iy)] + *value;
 					}
 				}
 			}
@@ -1153,7 +1172,7 @@ pub mod supernodal {
 					for (j_idx_in_c, j) in c_pattern.iter().enumerate() {
 						let j_idx_in_c = j_idx_in_c + c_ncols;
 						if j.zx() >= c_min_col {
-							s_H.write(s_idx, col_global_to_local[j.zx()].zx(), c_H.read(c_idx, j_idx_in_c));
+							s_H[(s_idx, col_global_to_local[j.zx()].zx())] = copy(c_H[(c_idx, j_idx_in_c)]);
 						}
 					}
 				}
@@ -1186,8 +1205,8 @@ pub mod supernodal {
 				let s_tau = &mut tau_val[tau_begin..tau_end];
 				let s_L = &mut L_val[L_begin..L_end];
 
-				let max_blocksize = H_symbolic.max_blocksize[s].zx();
-				let mut s_tau = MatMut::from_column_major_slice_mut(s_tau, max_blocksize, Ord::min(s_H.ncols(), s_h_row_full_end - s_h_row_begin));
+				let max_block_size = H_symbolic.max_block_size[s].zx();
+				let mut s_tau = MatMut::from_column_major_slice_mut(s_tau, max_block_size, Ord::min(s_H.ncols(), s_h_row_full_end - s_h_row_begin));
 
 				{
 					let mut current_min_col = 0usize;
@@ -1201,7 +1220,7 @@ pub mod supernodal {
 							s_H.ncols()
 						};
 
-						if idx_min_col == s_H.ncols() || idx_min_col >= current_min_col.saturating_add(Ord::max(1, max_blocksize / 2)) {
+						if idx_min_col == s_H.ncols() || idx_min_col >= current_min_col.saturating_add(Ord::max(1, max_block_size / 2)) {
 							let nrows = idx.saturating_sub(current_start);
 							let full_ncols = s_H.ncols() - current_start;
 							let ncols = Ord::min(nrows, idx_min_col - current_min_col);
@@ -1209,9 +1228,9 @@ pub mod supernodal {
 							let s_H = s_H.rb_mut().submatrix_mut(current_start, current_start, nrows, full_ncols);
 
 							let (mut left, mut right) = s_H.split_at_col_mut(ncols);
-							let bs = linalg::qr::no_pivoting::factor::recommended_blocksize::<Symbolic>(left.nrows(), left.ncols());
-							let bs = Ord::min(max_blocksize, bs);
-							tau_blocksize[block_count] = I::truncate(bs);
+							let bs = linalg::qr::no_pivoting::factor::recommended_block_size::<Symbolic>(left.nrows(), left.ncols());
+							let bs = Ord::min(max_block_size, bs);
+							tau_block_size[block_count] = I::truncate(bs);
 							householder_nrows[block_count] = I::truncate(nrows);
 							householder_ncols[block_count] = I::truncate(ncols);
 							block_count += 1;
@@ -1450,13 +1469,12 @@ pub mod simplicial {
 			self.tau_val
 		}
 
-		/// solves the equation $A x = \text{rhs}$ in the sense of least squares, implicitly
-		/// conjugating $A$ if needed
+		/// Applies $Q^{\top}$ to the input matrix `rhs`, implicitly conjugating the $Q$
+		/// matrix if needed
 		///
 		/// `work` is a temporary workspace with the same dimensions as `rhs`.
-		#[track_caller]
 		#[math]
-		pub fn solve_in_place_with_conj(&self, conj_qr: Conj, rhs: MatMut<'_, T>, par: Par, work: MatMut<'_, T>)
+		pub fn apply_qt_in_place_with_conj(&self, conj_qr: Conj, rhs: MatMut<'_, T>, par: Par, work: MatMut<'_, T>)
 		where
 			T: ComplexField,
 		{
@@ -1467,10 +1485,6 @@ pub mod simplicial {
 			let m = self.symbolic.nrows;
 			let n = self.symbolic.ncols;
 
-			let r = SparseColMatRef::<'_, I, T>::new(
-				unsafe { SymbolicSparseColMatRef::new_unchecked(n, n, self.r_col_ptr, None, self.r_row_idx) },
-				self.r_val,
-			);
 			let h = SparseColMatRef::<'_, I, T>::new(
 				unsafe { SymbolicSparseColMatRef::new_unchecked(m, n, self.householder_col_ptr, None, self.householder_row_idx) },
 				self.householder_val,
@@ -1513,12 +1527,37 @@ pub mod simplicial {
 			}
 			x.rb_mut().subrows_mut(0, n).copy_from(tmp.rb().subrows(0, n));
 			x.rb_mut().subrows_mut(n, m - n).fill(zero());
+		}
+
+		/// solves the equation $A x = \text{rhs}$ in the sense of least squares, implicitly
+		/// conjugating $A$ if needed
+		///
+		/// `work` is a temporary workspace with the same dimensions as `rhs`.
+		#[track_caller]
+		#[math]
+		pub fn solve_in_place_with_conj(&self, conj_qr: Conj, rhs: MatMut<'_, T>, par: Par, work: MatMut<'_, T>)
+		where
+			T: ComplexField,
+		{
+			let mut work = work;
+			let mut rhs = rhs;
+			self.apply_qt_in_place_with_conj(conj_qr, rhs.rb_mut(), par, work.rb_mut());
+
+			let _ = par;
+			assert!(rhs.nrows() == self.symbolic.nrows);
+			let mut x = rhs;
+
+			let n = self.symbolic.ncols;
+			let r = SparseColMatRef::<'_, I, T>::new(
+				unsafe { SymbolicSparseColMatRef::new_unchecked(n, n, self.r_col_ptr, None, self.r_row_idx) },
+				self.r_val,
+			);
 
 			linalg_sp::triangular_solve::solve_upper_triangular_in_place(r, conj_qr, x.rb_mut().subrows_mut(0, n), par);
 		}
 	}
 
-	/// computes the size and alignment of the workspace required to compute the symbolic $QR$
+	/// computes the layout of the workspace required to compute the symbolic $QR$
 	/// factorization of a matrix with dimensions `(nrows, ncols)`
 	pub fn factorize_simplicial_symbolic_qr_scratch<I: Index>(nrows: usize, ncols: usize) -> StackReq {
 		let _ = nrows;
@@ -1581,7 +1620,7 @@ pub mod simplicial {
 		})
 	}
 
-	/// computes the size and alignment of the workspace required to compute the numerical $QR$
+	/// computes the layout of the workspace required to compute the numerical $QR$
 	/// factorization of the matrix whose structure was used to produce the symbolic structure
 	pub fn factorize_simplicial_numeric_qr_scratch<I: Index, T: ComplexField>(symbolic: &SymbolicSimplicialQr<I>) -> StackReq {
 		let m = symbolic.nrows;
@@ -1719,8 +1758,8 @@ pub mod simplicial {
 
 			let (mut head, tail) = h_col.rb_mut().split_at_row_mut(1);
 			let head = &mut head[0];
-			let (tau, _) = crate::linalg::householder::make_householder_in_place(head, tail);
-			tau_val[j] = tau;
+			let crate::linalg::householder::HouseholderInfo { tau, .. } = crate::linalg::householder::make_householder_in_place(head, tail);
+			tau_val[j] = from_real(tau);
 			r_val[r_pos] = copy(*head);
 			*head = one();
 
@@ -1854,7 +1893,7 @@ impl<'a, I: Index, T> QrRef<'a, I, T> {
 			},
 			SymbolicQrRaw::Supernodal(symbolic) => {
 				let (householder_row_idx, indices) = indices.split_at(symbolic.householder().len_householder_row_idx());
-				let (tau_blocksize, indices) =
+				let (tau_block_size, indices) =
 					indices.split_at(symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes());
 				let (householder_nrows, indices) =
 					indices.split_at(symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes());
@@ -1869,7 +1908,7 @@ impl<'a, I: Index, T> QrRef<'a, I, T> {
 					supernodal::SupernodalQrRef::<'_, I, T>::new_unchecked(
 						symbolic,
 						householder_row_idx,
-						tau_blocksize,
+						tau_block_size,
 						householder_nrows,
 						householder_ncols,
 						r_val,
@@ -1883,7 +1922,7 @@ impl<'a, I: Index, T> QrRef<'a, I, T> {
 
 		for j in 0..k {
 			for (i, p) in inv.iter().enumerate() {
-				rhs.write(i, j, x.read(p.zx(), j));
+				rhs[(i, j)] = copy(&x[(p.zx(), j)]);
 			}
 		}
 	}
@@ -1936,7 +1975,7 @@ impl<I: Index> SymbolicQr<I> {
 		}
 	}
 
-	/// returns the size and alignment of the workspace required to solve the system $A x =
+	/// returns the layout of the workspace required to solve the system $A x =
 	/// \text{rhs}$ in the sense of least squares
 	pub fn solve_in_place_scratch<T>(&self, rhs_ncols: usize, par: Par) -> StackReq
 	where
@@ -1948,7 +1987,7 @@ impl<I: Index> SymbolicQr<I> {
 		})
 	}
 
-	/// computes the required workspace size and alignment for a numerical $QR$ factorization
+	/// computes the required workspace layout for a numerical $QR$ factorization
 	pub fn factorize_numeric_qr_scratch<T>(&self, par: Par, params: Spec<QrParams, T>) -> StackReq
 	where
 		T: ComplexField,
@@ -2010,7 +2049,7 @@ impl<I: Index> SymbolicQr<I> {
 			},
 			SymbolicQrRaw::Supernodal(symbolic) => {
 				let (householder_row_idx, indices) = indices.split_at_mut(symbolic.householder().len_householder_row_idx());
-				let (tau_blocksize, indices) =
+				let (tau_block_size, indices) =
 					indices.split_at_mut(symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes());
 				let (householder_nrows, indices) =
 					indices.split_at_mut(symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes());
@@ -2030,7 +2069,7 @@ impl<I: Index> SymbolicQr<I> {
 
 				supernodal::factorize_supernodal_numeric_qr::<I, T>(
 					householder_row_idx,
-					tau_blocksize,
+					tau_block_size,
 					householder_nrows,
 					householder_ncols,
 					r_val,
@@ -2346,13 +2385,13 @@ mod tests {
 		let mut householder_val = vec![0.0; symbolic.householder().len_householder_val()];
 		let mut tau_val = vec![0.0; symbolic.householder().len_tau_val()];
 
-		let mut tau_blocksize = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
+		let mut tau_block_size = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
 		let mut householder_nrows = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
 		let mut householder_ncols = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
 
 		supernodal::factorize_supernodal_numeric_qr::<I, f64>(
 			&mut householder_row_idx,
-			&mut tau_blocksize,
+			&mut tau_block_size,
 			&mut householder_nrows,
 			&mut householder_ncols,
 			&mut L_val,
@@ -2386,7 +2425,7 @@ mod tests {
 
 		let (m, n, col_ptr, row_idx, val) =
 			load_mtx::<usize>(MtxData::from_file(PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("test_data/sparse_qr/lp_share2b.mtx")).unwrap());
-		let val = val.iter().map(|&x| c64::new(x, gen.gen())).collect::<Vec<_>>();
+		let val = val.iter().map(|&x| c64::new(x, gen.random())).collect::<Vec<_>>();
 
 		let nnz = row_idx.len();
 
@@ -2459,13 +2498,13 @@ mod tests {
 		let mut householder_val = vec![T::ZERO; symbolic.householder().len_householder_val()];
 		let mut tau_val = vec![T::ZERO; symbolic.householder().len_tau_val()];
 
-		let mut tau_blocksize = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
+		let mut tau_block_size = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
 		let mut householder_nrows = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
 		let mut householder_ncols = vec![0usize; symbolic.householder().len_householder_row_idx() + symbolic.householder().n_supernodes()];
 
 		let qr = supernodal::factorize_supernodal_numeric_qr::<I, T>(
 			&mut householder_row_idx,
-			&mut tau_blocksize,
+			&mut tau_block_size,
 			&mut householder_nrows,
 			&mut householder_ncols,
 			&mut L_val,
@@ -2485,7 +2524,7 @@ mod tests {
 
 		let a = A.as_dyn().to_dense();
 
-		let rhs = Mat::<T>::from_fn(m, 2, |_, _| c64::new(gen.gen(), gen.gen()));
+		let rhs = Mat::<T>::from_fn(m, 2, |_, _| c64::new(gen.random(), gen.random()));
 		let mut x = rhs.clone();
 		let mut work = rhs.clone();
 		qr.solve_in_place_with_conj(
@@ -2517,7 +2556,7 @@ mod tests {
 		let (m, n, col_ptr, row_idx, val) =
 			load_mtx::<usize>(MtxData::from_file(PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("test_data/sparse_qr/lp_share2b.mtx")).unwrap());
 
-		let val = val.iter().map(|&x| c64::new(x, gen.gen())).collect::<Vec<_>>();
+		let val = val.iter().map(|&x| c64::new(x, gen.random())).collect::<Vec<_>>();
 
 		let nnz = row_idx.len();
 
@@ -2607,7 +2646,7 @@ mod tests {
 		);
 
 		let a = A.as_dyn().to_dense();
-		let rhs = Mat::<T>::from_fn(m, 2, |_, _| c64::new(gen.gen(), gen.gen()));
+		let rhs = Mat::<T>::from_fn(m, 2, |_, _| c64::new(gen.random(), gen.random()));
 		{
 			let mut x = rhs.clone();
 			let mut work = rhs.clone();
@@ -2647,7 +2686,7 @@ mod tests {
 
 		let (m, n, col_ptr, row_idx, val) =
 			load_mtx::<usize>(MtxData::from_file(PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("test_data/sparse_qr/lp_share2b.mtx")).unwrap());
-		let val = val.iter().map(|&x| c64::new(x, gen.gen())).collect::<Vec<_>>();
+		let val = val.iter().map(|&x| c64::new(x, gen.random())).collect::<Vec<_>>();
 		let nnz = row_idx.len();
 
 		let A = SparseColMatRef::<'_, I, T>::new(SymbolicSparseColMatRef::new_checked(m, n, &col_ptr, None, &row_idx), &val);
@@ -2668,7 +2707,7 @@ mod tests {
 		let (m, n) = (n, m);
 
 		let a = A.to_dense();
-		let rhs = Mat::<T>::from_fn(m, 2, |_, _| c64::new(gen.gen(), gen.gen()));
+		let rhs = Mat::<T>::from_fn(m, 2, |_, _| c64::new(gen.random(), gen.random()));
 
 		for supernodal_flop_ratio_threshold in [
 			SupernodalThreshold::FORCE_SUPERNODAL,
@@ -2736,7 +2775,7 @@ mod tests {
 		let a0_col_ptr = vec![0usize; 21];
 		let A0 = SparseColMatRef::<'_, I, T>::new(SymbolicSparseColMatRef::new_checked(40, 20, &a0_col_ptr, None, &[]), &[]);
 
-		let a1_val = [c64::new(gen.gen(), gen.gen()), c64::new(gen.gen(), gen.gen())];
+		let a1_val = [c64::new(gen.random(), gen.random()), c64::new(gen.random(), gen.random())];
 		let A1 = SparseColMatRef::<'_, I, T>::new(SymbolicSparseColMatRef::new_checked(40, 5, &[0, 1, 2, 2, 2, 2], None, &[0, 0]), &a1_val);
 		let A2 = SparseColMatRef::<'_, I, T>::new(SymbolicSparseColMatRef::new_checked(40, 5, &[0, 1, 2, 2, 2, 2], None, &[4, 4]), &a1_val);
 
