@@ -10,7 +10,7 @@
 // License for the specific language governing permissions and limitations
 // under the License.
 
-use ahash::RandomState;
+use foldhash::fast::RandomState;
 use priority_queue::PriorityQueue;
 use std::cmp::Ordering;
 use std::cmp::Reverse;
@@ -402,9 +402,9 @@ where
 ///
 /// * `graph` - The graph object to run the algorithm on
 /// * `preset_color_fn` - A callback function that will receive the node identifier
-///     for each node in the graph and is expected to return an `Option<usize>`
-///     (wrapped in a `Result`) that is `None` if the node has no preset and
-///     the usize represents the preset color.
+///   for each node in the graph and is expected to return an `Option<usize>`
+///   (wrapped in a `Result`) that is `None` if the node has no preset and
+///   the usize represents the preset color.
 ///
 /// # Example
 /// ```rust
@@ -468,9 +468,9 @@ where
 ///
 /// * `graph` - The graph object to run the algorithm on.
 /// * `preset_color_fn` - A callback function that will receive the node identifier
-///     for each node in the graph and is expected to return an `Option<usize>`
-///     (wrapped in a `Result`) that is `None` if the node has no preset and
-///     the usize represents the preset color.
+///   for each node in the graph and is expected to return an `Option<usize>`
+///   (wrapped in a `Result`) that is `None` if the node has no preset and
+///   the usize represents the preset color.
 /// * `strategy` - The greedy strategy used by the algorithm.
 ///
 /// # Example
@@ -581,9 +581,9 @@ where
 ///
 /// * `graph` - The graph object to run the algorithm on.
 /// * `preset_color_fn` - A callback function that will receive the edge identifier
-///     for each edge in the graph and is expected to return an `Option<usize>`
-///     (wrapped in a `Result`) that is `None` if the edge has no preset and
-///     the usize represents the preset color.
+///   for each edge in the graph and is expected to return an `Option<usize>`
+///   (wrapped in a `Result`) that is `None` if the edge has no preset and
+///   the usize represents the preset color.
 /// * `strategy` - The greedy strategy used by the algorithm.
 ///
 /// # Example
@@ -1017,7 +1017,7 @@ mod test_node_coloring {
         let graph = Graph::<(), (), Undirected>::new_undirected();
         let preset_color_fn = |_| Ok::<Option<usize>, Infallible>(None);
 
-        for strategy in vec![
+        for strategy in [
             ColoringStrategy::Degree,
             ColoringStrategy::Saturation,
             ColoringStrategy::IndependentSet,
@@ -1308,7 +1308,7 @@ mod test_node_coloring {
             path_graph(Some(7), None, || (), || (), false).unwrap();
         let preset_color_fn = |_| Ok::<Option<usize>, Infallible>(None);
 
-        for strategy in vec![
+        for strategy in [
             ColoringStrategy::Degree,
             ColoringStrategy::Saturation,
             ColoringStrategy::IndependentSet,
@@ -1326,7 +1326,7 @@ mod test_node_coloring {
             cycle_graph(Some(15), None, || (), || (), false).unwrap();
         let preset_color_fn = |_| Ok::<Option<usize>, Infallible>(None);
 
-        for strategy in vec![
+        for strategy in [
             ColoringStrategy::Degree,
             ColoringStrategy::Saturation,
             ColoringStrategy::IndependentSet,
@@ -1344,7 +1344,7 @@ mod test_node_coloring {
             heavy_hex_graph(7, || (), || (), false).unwrap();
         let preset_color_fn = |_| Ok::<Option<usize>, Infallible>(None);
 
-        for strategy in vec![
+        for strategy in [
             ColoringStrategy::Degree,
             ColoringStrategy::Saturation,
             ColoringStrategy::IndependentSet,
@@ -1362,7 +1362,7 @@ mod test_node_coloring {
             complete_graph(Some(10), None, || (), || ()).unwrap();
         let preset_color_fn = |_| Ok::<Option<usize>, Infallible>(None);
 
-        for strategy in vec![
+        for strategy in [
             ColoringStrategy::Degree,
             ColoringStrategy::Saturation,
             ColoringStrategy::IndependentSet,
