@@ -16,6 +16,7 @@ use std::error::Error;
 use std::fmt::{Display, Formatter};
 use std::hash::Hash;
 
+use hashbrown::hash_set::Entry;
 use hashbrown::{HashMap, HashSet};
 use std::fmt::Debug;
 use std::mem::swap;
@@ -58,7 +59,7 @@ impl<E: Error> Display for TopologicalSortError<E> {
                 write!(f, "At least one initial node is reachable from another")
             }
             TopologicalSortError::KeyError(ref e) => {
-                write!(f, "The key callback failed with: {:?}", e)
+                write!(f, "The key callback failed with: {e:?}")
             }
         }
     }
@@ -79,19 +80,19 @@ impl<E: Error> Error for TopologicalSortError<E> {}
 ///
 /// * `dag`: The DAG to get the topological sorted nodes from
 /// * `key`: A function that gets passed a single argument, the node id from
-///     `dag` and is expected to return a key which will be used for
-///     resolving ties in the sorting order.
+///   `dag` and is expected to return a key which will be used for
+///   resolving ties in the sorting order.
 /// * `reverse`: If `false`, perform a regular topological ordering.  If `true`,
-///     return the lexicographical topological order that would have been found
-///     if all the edges in the graph were reversed.  This does not affect the
-///     comparisons from the `key`.
+///   return the lexicographical topological order that would have been found
+///   if all the edges in the graph were reversed.  This does not affect the
+///   comparisons from the `key`.
 /// * `initial`: If given, the initial node indices to start the topological
-///     ordering from.  If not given, the topological ordering will certainly contain every node in
-///     the graph.  If given, only the `initial` nodes and nodes that are dominated by the
-///     `initial` set will be in the ordering.  Notably, any node that has a natural in degree of
-///     zero will not be in the output ordering if `initial` is given and the zero-in-degree node
-///     is not in it.  It is not supported to give an `initial` set where the nodes have even
-///     a partial topological order between themselves and `None` will be returned in this case
+///   ordering from.  If not given, the topological ordering will certainly contain every node in
+///   the graph.  If given, only the `initial` nodes and nodes that are dominated by the
+///   `initial` set will be in the ordering.  Notably, any node that has a natural in degree of
+///   zero will not be in the output ordering if `initial` is given and the zero-in-degree node
+///   is not in it.  It is not supported to give an `initial` set where the nodes have even
+///   a partial topological order between themselves and `None` will be returned in this case
 ///
 /// # Returns
 ///
@@ -347,7 +348,7 @@ where
 ///
 /// * `graph` - The graph to get the layers from
 /// * `first_layer` - A list of node ids for the first layer. This
-///     will be the first layer in the output
+///   will be the first layer in the output
 ///
 /// Will `panic!` if a provided node is not in the graph.
 /// ```
@@ -419,12 +420,11 @@ where
             self.first_iter = false;
             for node in &self.cur_layer {
                 if self.graph.to_index(*node) >= self.graph.node_bound() {
-                    panic!("Node {:#?} is not present in the graph.", node);
+                    panic!("Node {node:#?} is not present in the graph.");
                 }
                 if self.cycle_check.contains(node) {
                     return Some(Err(LayersError(format!(
-                        "An invalid first layer was provided: {:#?} appears more than once.",
-                        node
+                        "An invalid first layer was provided: {node:#?} appears more than once."
                     ))));
                 }
                 self.cycle_check.insert(*node);
@@ -435,18 +435,22 @@ where
         } else {
             for node in &self.cur_layer {
                 if self.graph.to_index(*node) >= self.graph.node_bound() {
-                    panic!("Node {:#?} is not present in the graph.", node);
+                    panic!("Node {node:#?} is not present in the graph.");
                 }
                 let children = self
                     .graph
                     .neighbors_directed(*node, petgraph::Direction::Outgoing);
                 let mut used_indices: HashSet<G::NodeId> = HashSet::new();
                 for succ in children {
-                    // Skip duplicate successors
-                    if used_indices.contains(&succ) {
-                        continue;
+                    match used_indices.entry(succ) {
+                        Entry::Occupied(_) => {
+                            // Skip duplicate successors
+                            continue;
+                        }
+                        Entry::Vacant(used_indices_entry) => {
+                            used_indices_entry.insert();
+                        }
                     }
-                    used_indices.insert(succ);
                     let mut multiplicity: usize = 0;
                     let raw_edges: G::EdgesDirected = self
                         .graph
@@ -498,20 +502,20 @@ where
 ///
 /// * `graph`: The DAG to find bicolor runs in
 /// * `filter_fn`: The filter function to use for matching nodes. It takes
-///     in one argument, the node data payload/weight object, and will return a
-///     boolean whether the node matches the conditions or not.
-///     If it returns ``true``, it will continue the bicolor chain.
-///     If it returns ``false``, it will stop the bicolor chain.
-///     If it returns ``None`` it will skip that node.
+///   in one argument, the node data payload/weight object, and will return a
+///   boolean whether the node matches the conditions or not.
+///   If it returns ``true``, it will continue the bicolor chain.
+///   If it returns ``false``, it will stop the bicolor chain.
+///   If it returns ``None`` it will skip that node.
 /// * `color_fn`: The function that gives the color of the edge. It takes
-///     in one argument, the edge data payload/weight object, and will
-///     return a non-negative integer, the edge color. If the color is None,
-///     the edge is ignored.
+///   in one argument, the edge data payload/weight object, and will
+///   return a non-negative integer, the edge color. If the color is None,
+///   the edge is ignored.
 ///
 /// # Returns:
 ///
 /// * `Vec<Vec<G::NodeId>>`: a list of groups with exactly two edge colors, where each group
-///     is a list of node data payload/weight for the nodes in the bicolor run
+///   is a list of node data payload/weight for the nodes in the bicolor run
 /// * `None` if a cycle is found in the graph
 /// * Raises an error if found computing the bicolor runs
 ///
@@ -660,9 +664,9 @@ where
 ///
 /// * `graph`: The DAG to collect runs from
 /// * `include_node_fn`: A filter function used for matching nodes. It takes
-///     in one argument, the node data payload/weight object, and returns a
-///     boolean whether the node matches the conditions or not.
-///     If it returns ``false``, the node will be skipped, cutting the run it's part of.
+///   in one argument, the node data payload/weight object, and returns a
+///   boolean whether the node matches the conditions or not.
+///   If it returns ``false``, the node will be skipped, cutting the run it's part of.
 ///
 /// # Returns:
 ///
@@ -1413,7 +1417,7 @@ mod test_collect_runs {
         let mut runs = collect_runs(&graph, |_| -> Result<bool, ()> { Ok(true) }).expect("Some");
 
         let run = runs.next();
-        assert!(run == None);
+        assert!(run.is_none());
 
         let runs = collect_runs(&graph, |_| -> Result<bool, ()> { Ok(true) }).expect("Some");
 
