@@ -19,10 +19,14 @@ mod all_shortest_paths;
 mod astar;
 mod bellman_ford;
 mod dijkstra;
+mod distance_matrix;
 mod k_shortest_path;
+mod single_source_all_shortest_paths;
 
 pub use all_shortest_paths::all_shortest_paths;
 pub use astar::astar;
 pub use bellman_ford::{bellman_ford, negative_cycle_finder};
 pub use dijkstra::dijkstra;
+pub use distance_matrix::distance_matrix;
 pub use k_shortest_path::k_shortest_path;
+pub use single_source_all_shortest_paths::single_source_all_shortest_paths;
